@@ -180,7 +180,7 @@ class AnnotationManager(object):
                 return [self.get_classname(classid) for classid, score in enumerate(distribution) if score > 0]
             else:
                 max_score = np.max(distribution)
-                return [self.get_classname(classid) for classid, score in enumerate(distribution) if score == max_score]
+                return [self.get_classname(classid) for classid, score in enumerate(distribution) if (score == max_score) and (max_score > eps)]
         elif obj_type == 'list-list':     # we assume this is a list of distribution
             distribution_list = obj
             return [self.get_classname(distribution) for distribution in distribution_list]
