@@ -16,6 +16,7 @@ from .utils import glob_recursively
 
 from tqdm import tqdm
 from pathlib import Path
+from pathlib2 import Path as Path2
 
 from numpy.lib.function_base import iterable
 from matplotlib.backends.backend_agg import FigureCanvasAgg
@@ -64,15 +65,28 @@ def image_resize(img_source, shape=None, factor=None, unique_check=False, interp
     return resized_image
 
 
-def to_gray_image(image):
-    """
-    transfer a 3 channel image to  a 2 channels one
-    """
-    if image.ndim > 2:
-        gray_image = cv2.cvtColor(image, cv2.COLOR_RGB2GRAY)
+def to_gray_image(img):
+    if isinstance(img, Path) or isinstance(img, Path2):
+        if img.exists():
+            gray_img = cv_rgb_imread(img, gray=True)
+            return gray_img
+        else:
+            raise FileNotFoundError(f"file not found {str(img)}")
+    elif isinstance(img, str):
+        if OS_exists(img):
+            gray_img = cv_rgb_imread(img, gray=True)
+            return gray_img
+        else:
+            raise FileNotFoundError(f"file not found {str(img)}")
+    elif isinstance(img, np.ndarray):
+        if img.ndim == 2:
+            return img
+        elif img.ndim == 3:
+            return cv2.cvtColor(img, cv2.COLOR_RGB2GRAY)
+        else:
+            raise RuntimeError(f"ndim of img is not 2 or 3 {img.shape}")
     else:
-        gray_image = image
-    return gray_image
+        raise RuntimeError(f"unknown data type {type(img)}")
 
 
 def to_colorful_image(image):
@@ -697,13 +711,16 @@ def get_polygons_from_mask(mask):
     return contours
 
 
-def draw_polygons_for_mask(mask, polygons, color=None):
+def draw_polygons(mask, polygons, color=None, thickness=5, fill=False):
 
     if color is None:
-        color = 1
+        color = [255, 255, 255]
     if np.array(polygons).ndim == 2:
         polygons = [polygons]
-    mask = cv2.fillPoly(mask, [np.array(p).astype('int32') for p in polygons], [color, color, color])
+    if fill:
+        mask = cv2.fillPoly(mask, [np.array(p).astype('int32') for p in polygons], color)
+    else:
+        mask = cv2.polylines(mask, [np.array(p).astype('int32') for p in polygons], True, color=color, thickness=thickness)
 
     return mask
 
