@@ -8,14 +8,16 @@
 #
 
 # 整合常用os操作
+import os.path as osp
 from os.path import basename as OS_basename
 from os.path import join as OS_join
 from os.path import exists as OS_exists
 from os.path import isdir as OS_isdir
 from os.path import dirname as OS_dirname
-import re
 from numpy.lib.function_base import iterable
 
+import pypinyin
+
 from pathlib2 import Path as Path2
 from pathlib import Path
 import PIL.Image as Image
@@ -23,7 +25,7 @@ import PIL.Image as Image
 import matplotlib.pyplot as plt
 
 from glob import glob
-
+import string
 import numpy as np
 import inspect
 import json
@@ -40,10 +42,12 @@ import shutil
 
 from tqdm import tqdm
 
+from collections import OrderedDict
 from functools import wraps
 from functools import reduce
 
 
+
 def get_list_from_list(data_list, call_back, absolutely=False):
     """[make a list through a input list,
     while the output list will collect the output of a function dealing with every item of the input list]
@@ -164,6 +168,17 @@ def timer_vvd(func):
     return wrapper
 
 
+def file_rename(srcFile, dstFile):
+    srcFile = str(srcFile)
+    dstFile = str(dstFile)
+    assert OS_exists(srcFile), f"srcFile {srcFile} not found"
+    try:
+        os.rename(srcFile, dstFile)
+    except Exception as e:
+        print(e)
+        print('rename file fail\r\n')
+
+
 def get_current_dir():
     """
     get current dir of the running file
@@ -312,13 +327,15 @@ def save_file_path_check(save_file_path, overwrite=False, verbose=False):
     return checked_save_file_path
 
 
-def encode_chinese_to_unicode(input_string):
+def encode_chinese_to_unicode(input_string, remove=False):
     '''
     将中文转换为 unicode #Uxxxx 形式
     '''
     unicode_string = ''
     for char in input_string:
         if ord(char) > 255:
+            if remove:
+                continue
             char = "%%U%04x" % ord(char)
         unicode_string += char
     unicode_string = unicode_string.replace('%', '#')
@@ -920,7 +937,7 @@ def remove_small_components(mask, area_thres):
     return labels < 0
 
 
-def encode_path(path, seg_num = 4, key_word=None, sep_char='@', root_path=None, with_suffix=None):
+def encode_path(path, seg_num = 4, key_word=None, sep_char='@', root_path=None, with_suffix=None, stem=False):
 
     if key_word is not None:
         pos = str(path).find(key_word)
@@ -941,11 +958,163 @@ def encode_path(path, seg_num = 4, key_word=None, sep_char='@', root_path=None,
 
     if with_suffix is not None:
         encoded_path = str(Path(str(encoded_path)).with_suffix('.' + with_suffix))
+    
+    if stem:
+        encoded_path = Path(encoded_path).stem
 
     return encoded_path
 
 
+def extract_file_to(root_dir, suffix, target_dir, default_level=3):
+    file_list = glob_recursively(root_dir, suffix)
+    desc = f"Extracting {suffix} file from {root_dir} to {target_dir}:"
+    for file_path in tqdm(file_list, desc=desc):
+        file_name = encode_path(file_path, seg_num=default_level)
+        save_path = Path(target_dir) / file_name
+        smart_copy(file_path, save_path)
+
+
+def hardlink(src, tar):
+    if osp.isdir(tar):
+        target_path = osp.join(tar, osp.basename(src))
+    else:
+        target_path = tar
+    if osp.exists(target_path):
+        return False
+    else:
+        if osp.exists(src):
+            os.link(src, target_path)
+        else:
+            return False
+    return True
+
+
+class ListOrderedDict(OrderedDict):
+    def __getitem__(self, key):
+        if is_integer(key):
+            key = list(self.keys())[key]
+        elif isinstance(key, slice):
+            slicedkeys = list(self.keys())[key]
+            lod_obj = type(self)()
+            for slicedkey in slicedkeys:
+                lod_obj[slicedkey] = self[slicedkey]
+            return lod_obj
+
+        return super().__getitem__(key)
+
+    def append(self, key, value):
+        if is_integer(key):
+            raise RuntimeError(f" integer key is not allowed in ListOrderedDict {key}.")
+        assert key not in self, f"cannot add existed key. {key}"
+        self[key] = value
+    
+    def __setitem__(self, key, value):
+        'od.__setitem__(i, y) <==> od[i]=y'
+        # Setting a new item creates a new link at the end of the linked list,
+        # and the inherited dictionary is updated with the new key/value pair.
+        if is_integer(key):
+            raise RuntimeError(f" integer key is not allowed in ListOrderedDict {key}.")
+        else:
+            return super().__setitem__(key, value)
+
+    def __delitem__(self, key):
+        'od.__delitem__(y) <==> del od[y]'
+        # Deleting an existing item uses self.__map to find the link which gets
+        # removed by updating the links in the predecessor and successor nodes.
+        if is_integer(key):
+            key = list(self.keys())[key]
+
+        return super().__delitem__(key)
+
+    def move_to_end(self, key, last=True):
+        '''Move an existing element to the end (or beginning if last is false).
+
+        Raise KeyError if the element does not exist.
+        '''
+        if is_integer(key):
+            key = list(self.keys())[key]
+
+        return super().move_to_end(key, last)
+
+    def __repr__(self):
+        'od.__repr__() <==> repr(od)'
+        if not self:
+            return '%s()' % (self.__class__.__name__,)
+        return '%s(%r)' % (self.__class__.__name__, list(self.items()))
+
+    def pop(self, key=-1):
+        '''od.pop(k[,d]) -> v, remove specified key and return the corresponding
+        value.  If key is not found, d is returned if given, otherwise KeyError
+        is raised.
+        '''
+        if is_integer(key):
+            if len(list(self.keys())) == 0:
+                print(f"pop a None from an empty ListOrderedDict.")
+                return None
+            key = list(self.keys())[key]
+        return super().pop(key)
+
+    def setdefault(self, key, default=None):
+        '''Insert key with a value of default if key is not in the dictionary.
+
+        Return the value for key if key is in the dictionary, else default.
+        '''
+        if is_integer(key):
+            key = list(self.keys())[key]
+        return super().setdefault(key, default)
+
+
+def is_valid_identifier(input_str):
+    ascii = string.ascii_letters+'_'
+    digits = string.digits
+    if not isinstance(input_str, str):
+        return False
+    if len(input_str) == 0:
+        return False
+    if input_str[0] not in ascii:
+        return False
+    for char in input_str:
+        if char not in ascii and char not in digits:
+            return False
+    return True
+
+
+def chinese_str_to_pinyin(input_str, join_char='_', remove=True):
+    pinyin_str =  pypinyin.pinyin(input_str, style=pypinyin.NORMAL)
+    pinyin_str_list = get_list_from_list(pinyin_str, lambda x: x[0])
+    pinyin_res_str = join_char.join(pinyin_str_list)
+    output_str = encode_chinese_to_unicode(pinyin_res_str, remove=remove)
+    return output_str
+
+
+def remove_chinese_for_file_names(root_dir):
+    dir_list, file_list = get_dir_file_list(root_dir, recursive=True)
+    file_name_changed = 0
+    for file_path in tqdm(file_list, desc=f"changing file names: "):
+        file_name = Path(file_path).name
+        changed_name = chinese_str_to_pinyin(file_name)
+        if file_name != changed_name:
+            dst_path = Path(file_path).parent / changed_name
+            file_rename(file_path, dst_path)
+            file_name_changed += 1
+    print(f"change {file_name_changed} file names.")
+    
+    dir_name_changed = 0
+    for dir_path in tqdm(dir_list[::-1], desc=f"changing directory names: "):
+        dir_name = Path(dir_path).name
+        changed_name = chinese_str_to_pinyin(dir_name)
+        if dir_name != changed_name:
+            dst_path = Path(dir_path).parent / changed_name
+            file_rename(dir_path, dst_path)
+            dir_name_changed += 1
+    print(f"change {dir_name_changed} dir names.")
+    pass
+
+
 if __name__ == '__main__':
+    print(chinese_str_to_pinyin('asdfd_（阿斯蒂芬）'))
+    
+    is_valid_identifier('')
     test_name = 'abc/sadf/gsdf.sadf.test'
     strong_printing(test_name)
     print(underline_connection())
